//
// SendDlg.cpp : message sending dialog implementation
//
//  1999 Rick York
//
//@tabs=4

#include "StdAfx.h"
#include "Resource.h"
#include "IpcDemo.h"
#include "CProfile.h"
#include "SendDlg.h"
#include "CElapsed.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// profile topics and keys

#define	TOPIC_SENDPOS	"SendPosition"
#define	KEY_POSX		"xpos"
#define	KEY_POSY		"ypos"


///////////////////////////////////////////////////////////////////////
// helper functions


typedef struct
	{
	HWND	hwnd;
	ccp		title;
	} FindWnd;


CALLBACK CheckWindowTitle( HWND hwnd, LPARAM lParam )
	{
	char	buffer[MAX_PATH];
	GetWindowText( hwnd, buffer, sizeof( buffer ) );

	FindWnd * fw = (FindWnd *)lParam;
	if( strcmp( buffer, fw->title ) == 0 )
		{
		fw->hwnd = hwnd;
		return FALSE;
		}
	return TRUE;
	}


HWND FindWinTitle( ccp title )
	{
	FindWnd	fw;
	fw.hwnd = 0;
	fw.title = title;
	
	EnumWindows( (WNDENUMPROC) CheckWindowTitle, (LPARAM) &fw );
	return fw.hwnd;
	}


///////////////////////////////////////////////////////////////////////
// CIpcSendDlg dialog

CIpcSendDlg::CIpcSendDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CIpcSendDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CIpcSendDlg)
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}


void CIpcSendDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CIpcSendDlg)
	DDX_Control(pDX, IDC_SENDCOLOR, m_cbFgColor);
	DDX_Control(pDX, IDC_SENDBGCLR, m_cbBgColor);
 	DDX_Control(pDX, IDC_SENDTEXT, m_ebText);
	DDX_Control(pDX, IDC_SENDLINE, m_stLine);
	DDX_Control(pDX, IDC_LINESPIN, m_sbSpin);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CIpcSendDlg, CDialog)
	//{{AFX_MSG_MAP(CIpcSendDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_EXIT, OnExit)
	ON_BN_CLICKED(IDC_SENDMSG, OnSend)
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


// CIpcSendDlg message handlers

BOOL CIpcSendDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// get and set the dialog location

	CProfile prf( theApp.m_IniFile );
	prf.SetTopic( TOPIC_SENDPOS );
	int wx = prf.GetInt( KEY_POSX, 200 );
	int wy = prf.GetInt( KEY_POSY, 200 );

	SetWindowPos( NULL, wx, wy, 0, 0, SWP_NOSIZE | SWP_NOZORDER );

	// set the title text

	CString title;
	title.LoadString( IDS_NAME_SEND );
	SetWindowText( title );

	// setup the controls

	m_ebText.SetWindowText( "" );
	m_ebText.LimitText( MSGCMD_TEXTSIZE );

	m_sbSpin.SetBuddy( &m_stLine );
	m_sbSpin.SetRange( 1, MSGCMD_MAXLINES );
	m_sbSpin.SetPos( 1 );

	m_cbFgColor.SetCurSel( 0 );
	m_cbBgColor.SetCurSel( 1 );

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if( pSysMenu != NULL )
		{
		CString strTimeTest;
		strTimeTest.LoadString(IDS_TIMETEST);
		if( ! strTimeTest.IsEmpty() )
			{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_TIMETEST, strTimeTest);
			}
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if( ! strAboutMenu.IsEmpty() )
			{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
			}
		}

	// Set the icon for this dialog.
	// The framework does this automatically when
	// the app's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// start up a receiver

	char appname[MAX_PATH];
	GetModuleFileName( AfxGetInstanceHandle(),
					appname, sizeof( appname ) );

	STARTUPINFO	sui;
	int size = sizeof( sui );
	memset( & sui, 0, size );
	sui.cb = size;

	PROCESS_INFORMATION pi;
	size = sizeof( pi );
	memset( & pi, 0, size );

	BOOL bTest = CreateProcess(
						appname,	// executable image
						NULL,		// command line
						NULL,		// process security
						NULL,		// thread security
						FALSE,		// inherit handles
						0,			// creation flags
						NULL,		// environment block
						NULL,		// current directory
						&sui,		// startup info
						&pi );		// process info

	BOOL error = FALSE;
	if( ! bTest )
		error = TRUE;
	else
		{
		// wait for it to start up

		if( WaitForInputIdle( pi.hProcess, 5000 ) != 0 )
			error = TRUE;	// failed
		CloseHandle( pi.hProcess );
		CloseHandle( pi.hThread );
		}

	if( error )
		{
		MessageBox( "Failed Starting Receiver", "Error" );
		EndDialog( IDCANCEL );
		}

	// get a handle to the receiver

	CString rcvtitle;
	rcvtitle.LoadString( IDS_NAME_RECV );
	m_hWndRecv = FindWinTitle( rcvtitle );
	if( ! m_hWndRecv )
		{
		MessageBox( "Failed Finding Receiver Window", "Error" );
		EndDialog( IDCANCEL );
		}

	return TRUE;
}


void CIpcSendDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if( (nID & 0xFFF0) == IDM_ABOUTBOX )
		{
		CDialog dlgAbout( IDD_ABOUTBOX );
		dlgAbout.DoModal();
		}
	else if( (nID & 0xFFF0) == IDM_TIMETEST )
		{
		CElapsed et;
		if( ! et.IsValid() )
			{
			MessageBox( "Performance Counter is not available",
						"Sorry", MB_OK | MB_ICONSTOP );
			return;
			}
		et.Begin();
		int loops = 10000;
		for( int i = 0; i < loops; i += 1 )
			SendIpcMsg( MSGCMD_TIME, 0, 0, 0, NULL );
		double elapsed = et.End();
		char buf[80];
		sprintf( buf, "Elapsed time for %d loops was %.3f seconds",
				loops, elapsed );
		MessageBox( buf, "Results", MB_OK );
		}
	else
		{
		CDialog::OnSysCommand(nID, lParam);
		}
}


// If you add a minimize button to your dialog,
// you will need the code below to draw the icon.  

void CIpcSendDlg::OnPaint() 
{
	if( IsIconic() )
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display
// while the user drags the minimized window.
HCURSOR CIpcSendDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}


void CIpcSendDlg::OnClose() 
{
	TRACE( "CIpcSendDlg::OnClose\n" );
	OnExit();
}


void CIpcSendDlg::OnExit() 
{
	// terminate receiver

	SendIpcMsg( MSGCMD_EXIT, 0, 0, 0, "Exit" );

	// save window position

	CProfile prf( theApp.m_IniFile );
	prf.SetTopic( TOPIC_SENDPOS );
	RECT rect;
	GetWindowRect( &rect );
	int wx = max( rect.left, 0 );
	int wy = max( rect.top, 0 );

	prf.PutInt( KEY_POSX, wx );
	prf.PutInt( KEY_POSY, wy );

	// end of the line
	
	EndDialog( IDOK );
}


void CIpcSendDlg::OnSend() 
{
	// get the data

	int line = m_sbSpin.GetPos();
	COLORREF fgclr = m_cbFgColor.GetSelectedColorValue();
	COLORREF bgclr = m_cbBgColor.GetSelectedColorValue();
	CString txt;
	m_ebText.GetWindowText( txt );

	// send the message

	SendIpcMsg(	MSGCMD_TEXT, line, fgclr, bgclr, txt );
}


int CIpcSendDlg::SendIpcMsg( int cmd, int line, COLORREF fgclr,
							COLORREF bgclr, ccp txt )
{
	MsgCmd	msg;

	msg.command = cmd;
	if( cmd != MSGCMD_TIME )
		{
		msg.line = line;
		msg.fgcolor = fgclr;
		msg.bgcolor = bgclr;
		strcpy( msg.text, txt );
		}

	COPYDATASTRUCT	cds;

	cds.dwData = 0;
	cds.cbData = sizeof( msg );
	cds.lpData = &msg;

	return ::SendMessage( m_hWndRecv, WM_COPYDATA,
						(WPARAM) m_hWnd, (LPARAM) &cds );
}

